/*
 Copyright (c) 2007 Noodlesoft, L.L.C.
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:
 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.
 
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
*/

#include <CoreFoundation/CoreFoundation.h>
#include <CoreAudio/CoreAudio.h>
#include <asl.h>

static AudioDeviceID		_previousDeviceID;

OSStatus getDataSource(AudioDeviceID deviceID, UInt32 *dataSource)
{
	UInt32		size;
	OSStatus	status;
	
	size = sizeof(UInt32);
	status = AudioDeviceGetProperty(deviceID, 0, 0, kAudioDevicePropertyDataSource, &size, dataSource);
	
	return status;
}

OSStatus getDefaultOutputDevice(AudioDeviceID *outDeviceID)
{
	OSStatus		status;
	UInt32			size;
	
	size = sizeof(AudioDeviceID);
	
	status = AudioHardwareGetProperty(kAudioHardwarePropertyDefaultOutputDevice, &size, outDeviceID);
	
	return status;
}

OSStatus setDefaultOutputDevice(AudioDeviceID deviceID)
{
	OSStatus		status;
	UInt32			size;
	
	size = sizeof(AudioDeviceID);
	
	status = AudioHardwareSetProperty(kAudioHardwarePropertyDefaultOutputDevice, size, &deviceID);
	
	return status;
}

OSStatus devicePropertyChanged(AudioDeviceID deviceID, UInt32 inChannel, Boolean isInput, AudioDevicePropertyID inPropertyID,
							   void *inClientData)
{
	OSStatus		status;
	UInt32			dataSource;
	AudioDeviceID	currentDeviceID;
	
	status = getDefaultOutputDevice(&currentDeviceID);
	
	if (status != noErr)
	{
		asl_log(NULL, NULL, ASL_LEVEL_ERR, "Error getting current output device: %d\n");
		exit(1);
	}
	
	status = getDataSource(deviceID, &dataSource);
	if (status != noErr)
	{
		asl_log(NULL, NULL, ASL_LEVEL_ERR, "Error getting data source: %d\n", status);
	}
	
	if (dataSource == 'hdpn')
	{
		if (currentDeviceID != deviceID)
		{
			asl_log(NULL, NULL, ASL_LEVEL_DEBUG, "Switching to headphones.\n");
			_previousDeviceID = currentDeviceID;
			setDefaultOutputDevice(deviceID);
		}
	}
	else if (dataSource == 'ispk')
	{
		if (currentDeviceID == deviceID)
		{
			asl_log(NULL, NULL, ASL_LEVEL_DEBUG, "Switching to previous output.\n");
			setDefaultOutputDevice(_previousDeviceID);
		}
	}
	
	return noErr;
}

void initSpeakerHeadphoneDevice()
{
	OSStatus		status;
	UInt32			size, dataSource;
	AudioDeviceID	deviceID, *deviceList;
	int				i;
	
	size = 0;
	status = AudioHardwareGetPropertyInfo(kAudioHardwarePropertyDevices, &size, NULL);
	if (status != noErr)
	{
		asl_log(NULL, NULL, ASL_LEVEL_ERR, "Error getting output device info: %d\n", status);
		exit(1);
	}
	
	deviceList = (AudioDeviceID *)malloc(size);
	
	status = AudioHardwareGetProperty(kAudioHardwarePropertyDevices, &size, deviceList);
	if (status != noErr)
	{
		asl_log(NULL, NULL, ASL_LEVEL_ERR, "Error getting output devices: %d\n", status);
		exit(1);
	}

	i = size / sizeof(AudioDeviceID);
	while (i--)
	{
		status = getDataSource(deviceList[i], &dataSource);
		if (status == noErr)
		{
			if ((dataSource == 'hdpn') || (dataSource == 'ispk'))
			{
				if ((dataSource == 'hdpn') && (_previousDeviceID != deviceList[i]))
				{
					status = setDefaultOutputDevice(deviceList[i]);
					
					if (status != noErr)
					{
						asl_log(NULL, NULL, ASL_LEVEL_ERR, "Error setting current output device: %d\n", status);
						exit(1);
					}
				}
				
				status = AudioDeviceAddPropertyListener(deviceList[i], 0, 0, kAudioDevicePropertyDataSource,devicePropertyChanged, NULL);
				
				if (status != noErr)
				{
					asl_log(NULL, NULL, ASL_LEVEL_ERR, "Error adding listener for device %d: %d", deviceID, status);
					exit(1);
				}
				
			}
		}		

	}
}

int main (int argc, const char * argv[])
{
	OSStatus		status;

	status = getDefaultOutputDevice(&_previousDeviceID);
	
	if (status != noErr)
	{
		asl_log(NULL, NULL, ASL_LEVEL_ERR, "Error getting current output device: %d", status);
		return 1;
	}
	
	initSpeakerHeadphoneDevice();
	
	CFRunLoopRun();
	
    return 0;
}
